#!/usr/bin/python3
# -*- coding: utf-8 -*-
##\namespace 

import sys
import os
import shutil
import string
import pwd
import subprocess
import string
import ldap
import subprocess
from subprocess import Popen
from pwd import getpwnam
from PyQt5 import QtCore, QtGui, QtWidgets

from ui_changePass import Ui_MainWindow
from cgaldapsearch import LDAP, PasswordIncorrect


class CgaPasswordError(Exception): pass 
class CgaEmptyFields(Exception): pass
class CgaInvalidCharacter(Exception): pass


VALID_CHARS = string.ascii_letters + "0123456789"
VALID_CHARS_EXTENDED = VALID_CHARS + "ñ" + "áéíóú" + "ç" + "-_." + " "


class SChangePass(QtWidgets.QMainWindow, Ui_MainWindow):
	
	def __init__(self, parent = None):
		super(SChangePass, self).__init__(parent)
		self.settings = QtCore.QSettings()
		self.setupUi(self)
		self.signals()
		self.t_pass_act.setEchoMode(QtWidgets.QLineEdit.Password)


	def signals(self):
		self.b_cancelar.clicked.connect(self.close)
		self.b_aceptar.clicked.connect(self.changePassword)
		self.checkPass.stateChanged.connect(self.passVisible)
	

	def changePassword(self):
		login = str(self.t_id.text())
		pwdAct = str(self.t_pass_act.text())
		pwd1 = str(self.t_pwd1.text())
		pwd2 = str(self.t_pwd2.text())
		mensaje = None

		try:
			self.data_ok(login, pwdAct, pwd1, pwd2)
			self.is_sane(pwdAct)
			self.is_sane(pwd1)
			self.is_sane(pwd2)
			self.is_sane(login, restricted = True)			

			if login[0].isupper():
				self.msgbox("El nombre del usuario no es válido", "critical")
				return

		except CgaInvalidCharacter as e:
			mensaje = str(""" Uno de los campos contiene caracteres no válidos. Los caracteres válidos para cada campo son:
						*Contraseña:
						'%s'
						*Login:
						'%s' """ % (VALID_CHARS_EXTENDED, VALID_CHARS))
			self.msgbox(mensaje, "warning")

		except CgaEmptyFields:
			mensaje = "Todos los campos son obligatorios. \nPor favor compruebe que ha rellenado todos los campos."
		except CgaPasswordError:
			mensaje = "Las contraseñas son distintas. \nPor favor asegúrese de que escribe la contraseña correctamente en los dos campos."

		if mensaje is not None:
			self.msgbox(mensaje, "critical")
			return

		self.loginn()

		p = LDAP(login)

		try:
			p.passwd(pwdAct, pwd1)
			self.changePassLocal(login, pwd1)

			if p.cp == True and self.cpl == True:
				msg = "Contraseña de Ldap y usuario local actualizada correctamente."
				self.msgbox(msg, "information")

			elif p.cp == True and self.cpl == False:
				msg = "Contraseña de Ldap actualizada correctamente."
				self.msgbox(msg, "information")

			self.reset_gui()
			self.hide()

		except PasswordIncorrect:
			msg = "La contraseña actual no coincide."
			self.msgbox(msg, "critical")
			return False

		except Exception as e:
			print("Exception: %s " %e)
			msg = "No se ha podido actualizar la contraseña. \n Intentelo de nuevo más tarde."
			self.msgbox(msg, "critical")
			return False



	def changePassLocal(self, login, pwd1):
		try:
			user = pwd.getpwnam(login)
			comando = ('echo "{username}:{password}" | chpasswd'.format(
									username=login,
									password=pwd1))

			(ret, salida)=subprocess.getstatusoutput(comando)
			comando=None
			print("Contraseña del usuario local actualizada")
			self.cpl = True

			if ret!=0:
				self.cpl = False
				msg = "Error al actualizar la contraseña del usuario local."
				self.msgbox(msg, "critical")

		except:
			self.cpl = False
			print("El usuario local no existe.")



	def loginn(self):
		ficheroentrada = open('/etc/resolv.conf','r')
		palabra = "search"
		for linea in ficheroentrada:
			if linea[0] != "#":
				if palabra in linea:
					domain = linea.split(" ")[1].split(".")[0]
					provincia=linea.split(" ")[1].split(".")[1]
		ficheroentrada.close()

		self.id = str(self.t_id.text()).strip()
		self.password = str(self.t_pass_act.text()).strip()

		if self.id != " " and self.password != " " :
			LDAP_SERVER = "ldap://c0:389"
			l = ldap.initialize(LDAP_SERVER)
			username = "uid=%s,ou=People,dc=%s,dc=%s,dc=andared,dc=ced,dc=junta-andalucia,dc=es" % (self.id, domain, provincia)
			password = self.password

			try:
				l.simple_bind_s(username, password)


			except ldap.INVALID_CREDENTIALS:
				msg = "Credenciales inválidas. Inténtelo de nuevo."
				self.msgbox(msg, "critical")

			except ldap.SERVER_DOWN:
				msg = "No estás conectado a un centro. Inténtelo de nuevo."
				self.msgbox(msg, "critical")

			except Exception as e:
				msg = "Error desconocido. Inténtelo de nuevo más tarde."
				self.msgbox(msg, "critical")
				print(e)



	def msgbox(self, msg, tipo):
		if tipo == "information":
			QtWidgets.QMessageBox.information(self, "Información", msg)
		elif tipo == "critical":
			QtWidgets.QMessageBox.critical(self, "Error", msg, QtWidgets.QMessageBox.Ok)


	def data_ok(self, login, pwdAct, pwd1, pwd2):
		if login == "" or pwdAct == "" or pwd1 == "" or pwd2 == "":
			raise CgaEmptyFields
		if pwd1 != pwd2:
			raise CgaPasswordError


	def is_sane(self, cad, restricted=False):
		valid = (VALID_CHARS_EXTENDED, VALID_CHARS)[restricted]
		res = [k in valid for k in cad]
		if not all(res):
			raise CgaInvalidCharacter(cad)


	def passVisible(self):
		if self.checkPass.isChecked():
			self.t_pass_act.setEchoMode(QtWidgets.QLineEdit.Normal)
			self.t_pwd1.setEchoMode(QtWidgets.QLineEdit.Normal)
			self.t_pwd2.setEchoMode(QtWidgets.QLineEdit.Normal)
		else:
			self.t_pass_act.setEchoMode(QtWidgets.QLineEdit.Password)
			self.t_pwd1.setEchoMode(QtWidgets.QLineEdit.Password)
			self.t_pwd2.setEchoMode(QtWidgets.QLineEdit.Password)

	
	def reset_gui(self):
		self.t_id.setText("")
		self.t_pass_act.setText("")
		self.t_pwd1.setText("")
		self.t_pwd2.setText("")
		self.checkPass.setChecked(False)


	def close(self):
		self.hide()



if __name__ == '__main__':
	app = QtWidgets.QApplication(sys.argv)
	c = SChangePass()
	c.show()
	app.exec_()
		
