#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import shutil
import string
import pwd
import subprocess
import string
from subprocess import Popen
from pwd import getpwnam
from PyQt5 import QtGui, QtCore, QtWidgets

from ui_configUser import Ui_MainWindow
from cgauser import *
from cgauserconfig import *


class CgaPasswordError(Exception): pass 
class CgaEmptyFields(Exception): pass
class CgaInvalidCharacter(Exception): pass


VALID_CHARS = string.ascii_letters + "0123456789"
VALID_CHARS_EXTENDED = VALID_CHARS + "ñ" + "áéíóú" + "ç" + "-_." + " "


class SConfigUser(QtWidgets.QMainWindow, Ui_MainWindow):
	
	def __init__(self, parent = None):
		super(SConfigUser, self).__init__(parent)
		self.setupUi(self)
		self.settings = QtCore.QSettings()
		self.signals()


	def signals(self):
		self.b_aceptar.clicked.connect(self.create_user)
		self.b_cancelar.clicked.connect(self.close)
		self.checkPass.stateChanged.connect(self.passVisible)


	def create_user(self):
		usuario = str(self.texto_usuario.text())
		login = str(self.texto_id.text())
		pwd1 = str(self.texto_pwd1.text())
		pwd2 = str(self.texto_pwd2.text())
		mensaje = None
		

		try:
			self._data_ok(usuario, login, pwd1, pwd2)
			self.is_sane(usuario)
			self.is_sane(pwd1)
			self.is_sane(pwd2)
			self.is_sane(login, restricted = True)
			

			if login[0].isupper():
				self.msgbox("El nombre del usuario no es válido", "critical")
				return

		except CgaInvalidCharacter as e:
			mensaje = str(""" Uno de los campos contiene caracteres no válidos. Los caracteres válidos para cada campo son:
						*Nombre y Contraseña:
						'%s'
						*Login:
						'%s' """ % (VALID_CHARS_EXTENDED, VALID_CHARS))
			self.msgbox(mensaje, "warning")
		except CgaEmptyFields:
			mensaje = "Todos los campos son obligatorios.\nPor favor compruebe que ha rellenado todos los campos."
		except CgaPasswordError:
			mensaje = "Las contraseñas son distintas.\nPor favor asegúrese de que escribe la contraseña correctamente en los dos campos."
		
		if mensaje is not None:
			self.msgbox(mensaje, "critical")
			return

		error = None
		try:
			CgaUser(login, usuario, pwd1).checkUser()

		except CgaUserExists as msg:
			print("El usuario ya existe")
			error = msg
		except CgaUserCreationError as msg:
			print("Ha ocurrido un error al crear el usuario")
			error = msg

		if error:
			msg =  "No se pudo crear el usuario '%s': \n %s" % (login, error)
			self.msgbox(msg, "critical")
		else:
			try:
				self.functionTic(str(login))
				msg = "Usuario '%s' configurado correctamente. \nSi es la primera vez que configura la sesión, los datos del sistema antiguo habrán sido sincronizados." % login
				self.msgbox(msg, "information")
				self.reset_gui()
				self.hide()

			except Exception as e:
				print("Error al crear el usuario %s " %e)

			
	def functionTic(self, login):
		uid = getpwnam(login).pw_uid
		gid = getpwnam(login).pw_gid
		HOMEDIR = os.path.join("/home/" + login + os.path.sep)
		fichero = 'nextcloud.cfg'
		ficheroAuto = 'TicBoxCloud.desktop'
		ruta = os.path.join(HOMEDIR + '.local/share/data/TicBoxCloud')
		ruta2 = os.path.join(HOMEDIR + '.local')
		rutaAuto = os.path.join(HOMEDIR + '.config/autostart')
		rutaAuto2 = os.path.join(rutaAuto + os.path.sep + ficheroAuto)

		if os.path.exists(HOMEDIR):
                    process = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), HOMEDIR])

		if not os.path.exists(rutaAuto):
			os.makedirs(rutaAuto, 755, exist_ok=True)

		sentCpAuto = "cp '"+ficheroAuto+"' "+rutaAuto
		os.system(sentCpAuto)
		process = subprocess.Popen(['chmod', '0775', rutaAuto])
		process = subprocess.Popen(['chmod', '0661', rutaAuto2])
		process = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), rutaAuto])

		if not os.path.exists(ruta):
			os.makedirs(ruta, mode = 0o775)

		sentCp = "cp '"+fichero+"' "+ruta
		os.system(sentCp)
		lineas = ["0\Folders\TicBoxCloud\localPath=" + HOMEDIR + "TicBoxCloud"+ os.path.sep, "0\http_user=" + login, "0\\user=" + login]
		path = os.path.join(ruta,str(fichero))
		if os.path.exists(os.path.abspath(path)):
			self.archivo = open(path, 'a+')
			for l in lineas:
				self.archivo.write(l + '\n')
			self.archivo.close()
			process = subprocess.Popen(['chmod', '-R', '0775', ruta2])
			proceso = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), ruta2])

		pathTic = os.path.join(HOMEDIR + "TicBoxCloud")
		if not os.path.exists(os.path.abspath(pathTic)):
			os.mkdir(pathTic, 0o700)
			process = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), pathTic])


	def msgbox(self, msg, tipo):
		if tipo == "information":
			QtWidgets.QMessageBox.information(self, "Información", msg)
		elif tipo == "critical":
			QtWidgets.QMessageBox.critical(self, "Error", msg, QtWidgets.QMessageBox.Ok)


	def _data_ok(self, usuario, login, pwd1, pwd2):
		if usuario == "" or login == "" or pwd1 == "" or pwd2 == "":
			raise CgaEmptyFields
		if pwd1 != pwd2:
			raise CgaPasswordError


	def is_sane(self, cad, restricted=False):
		valid = (VALID_CHARS_EXTENDED, VALID_CHARS)[restricted]
		res = [k in valid for k in cad]
		if not all(res):
			raise CgaInvalidCharacter(str(cad))


	def passVisible(self):
		if self.checkPass.isChecked():
			self.texto_pwd1.setEchoMode(QtWidgets.QLineEdit.Normal)
			self.texto_pwd2.setEchoMode(QtWidgets.QLineEdit.Normal)
		else:
			self.texto_pwd1.setEchoMode(QtWidgets.QLineEdit.Password)
			self.texto_pwd2.setEchoMode(QtWidgets.QLineEdit.Password)


	def reset_gui(self):
		self.texto_id.setText("")
		self.texto_usuario.setText("")
		self.texto_pwd1.setText("")
		self.texto_pwd2.setText("")
		self.checkPass.setChecked(False)


	def close(self):
		self.hide()


if __name__ == '__main__':
	app = QtWidgets.QApplication(sys.argv)
	c = SConfigUser()
	c.show()
	app.exec_()
		
